// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <stdint.h>
#include <malloc.h>
#include "../display/tileset.h"
#include "../display/primitives/quad.h"

typedef uint8_t tileflag_t;
typedef uint8_t tileid_t;

typedef struct {
  tileid_t id;
  void *data;
  int32_t indiceStart, verticeStart;

  //todo: do I need to store the position within the prim where my verts are, or
  //should that be apart of the data?
} tile_t;

typedef struct {
  tileflag_t flags;
  int32_t indiceCount, verticeCount;
} tiledef_t;

typedef struct {
  tiledef_t *tileDefinitions;
  tileset_t *tileset;
} tilemap_t;

#define TILE_NULL 0
#define TILE_FLAG_DYNAMIC (tileflag_t)1

/**
 * Creates a tilemap from a tileset.
 * 
 * @param tileset Tileset to create from.
 * @returns The tilemap.
 */
tilemap_t * tileMapCreate(tileset_t *tileset);

/**
 * Destroys a previously created tilemap.
 * 
 * @param tilemap Tilemap to dispose.
 */
void tileMapDispose(tilemap_t *tilemap);